/*
 *
 ******************************* Creazione file *******************************
 *-------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco               Data di creazione : 19/07/1999
 * Codice   :
 *
 *
	MKDL 1.0  19/7/99

	Multi-pattern directory creator.

	Usage: MKDL32 [options] [d:][path]filename [&|!] [filename] ...

	options:

 	-C[hh] set separator char (default: \t)
    -D try to create parent directory if fail


   Revision ported in win32 and stl


   Questa versione e' compilabile con:
   -    Borland C++ 4.X & STL HP (self modificate per funzionare senza warning e duplicazioni di funzioni)
   -    Borland C++ 5.X e standard C++ library fornite direttamente con il compilatore
   -    Borland C++ Builder 3 e standard C++ library fornite direttamente con il compilatore
   -    Microsoft Visual C++ 4.X e standard C++ library fornite dal compilatore
   -    Microsoft Visual C++ 5.X e standard C++ library fornite dal compilatore
   -    Microsoft Visual C++ 6.X e standard C++ library fornite dal compilatore

   Le condizioni in cui ho sviluppato il programma:

	BC++ 4.53 su Windows for Workgroup 3.11 & Win32s 1.30.166
	BC++ 5.x & VC++ 4.x su Windows NT 4.0
	BC++ 5.x & VC++ 5 SP3 su Windows NT 4.0 SP3

    Marco Savegnago (msave@tin.it)

	Luglio 1999
 ******************************* Revisione file *******************************
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

//	RTL include

#ifdef	__BORLANDC__
#include <dir.h>
#include <iostream.h>
#include <iomanip.h>
#include	<fstream.h>
#endif

#ifdef	_MSC_VER
#include <direct.h>
#include	<windows.h>
#include	<iostream>
#include	<iomanip>
#include	<fstream>
#endif

#include	<stdio.h>
#include	<ctype.h>
#include	<dos.h>
#include	<io.h>
#include	<errno.h>
#include	<conio.h>
#include	<stdlib.h>

//	STL include

#if defined(__BORLANDC__)
	#if 	(__BORLANDC__ <= 0x462)  // Borland C++ fino alla 4.53

	#include	<cstring.h>
	#include	<stl\list.h>
	#include	<stl\algo.h>

	typedef	list<string>						stringList;
   typedef	list<string>::iterator		   stringListIterator;

	#else

	#include <list>
	#include	<string>

	using namespace std;

	typedef	list<string>						stringList;
   typedef	list<string>::iterator		   stringListIterator;

	#endif

#elif defined (_MSC_VER)		//	VC++ & BC++ accettano gli header files senza estensione
#include	<string>
#include	<list>
#include <algorithm>

using namespace std;

typedef	allocator<string>					stringAllocator;
typedef	list<string, stringAllocator>	stringList;
typedef	list<string>::iterator			stringListIterator;

#endif

#define	MAX_LINELEN	1024

/******************* Prototipi delle funzioni di servizio *********************
*--------------------------------------|--------------------------------------*/
static	BOOL __stdcall ctrlBreakHandler( DWORD dw_CtrlType );
static	void usage();
static	int htoi(char *s);
static	char *strltrim(char *str);
static	char *strrtrim(char *str);
static	char *strtrim(char *str);
static	bool	createTree(char* pcsz_fileName);
static	char*	removeSeparator(char	*psz_line);
static	bool	readFile(const char* pcsz_fileList, bool	b_tryToCreateParent);

/****************************** Variabili locali ******************************
*--------------------------------------|--------------------------------------*/

char	sz_separator[]	=	"\t";

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco               Data di creazione : 19/07/1999
 * Codice   :
 *
 * Note: Intercetta i ctrl-c e ctrl-break
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

static	BOOL __stdcall ctrlBreakHandler( DWORD dw_CtrlType )
{
	if( dw_CtrlType == CTRL_C_EVENT || dw_CtrlType == CTRL_BREAK_EVENT )
	{
		cerr << ("Ctrl-Break pressed! Program aborted");
      ExitProcess(2);
		return TRUE;
	}
	else
		  return FALSE; // Don't handle 'CLOSE', 'LOGOFF' or 'SHUTDOWN' events.
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco               Data di creazione : 19/07/1999
 * Codice   :
 *
 * Note:	print usage msg to stderr when the args in command line is less than 2
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

static	void usage()
{
    cerr
        << "\n"
        << "MKDL (Win32) 1.0 " __DATE__ "\n\n"

        << "Multi-Directory creator\n\n"

        << "Usage: MKDL [options] <dirlist>\n"

        << "options:\n\n"

        << "  -C[hh] set separator char (default: 0x09 (TAB), -1 to disable)\n"
        << "  -D try to create parent directory if fail\n"

        << endl
	;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco               Data di creazione : 19/07/1999
 * Codice   :
 *
 * Note: Trasforma una stringa che rappresenta un numero hex in int
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

static	int htoi(char *s)
{
	int i = 0;
	char c;

	while((c = *s++) != '\0')
	{
		if(c == 'x')
			continue;   /* allow 0x notation */
		if('0' <= c && c <= '9')
			i = (i * 16) + (c - '0');
		else if('a' <= c && c <= 'f')
			i = (i * 16) + (c - 'a' + 10);
		else if('A' <= c && c <= 'F')
			i = (i * 16) + (c - 'A' + 10);
		else
			break;
	}
	return i;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco               Data di creazione : 19/07/1999
 * Codice   :
 *
 * Note:	strip space on left
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

static	char *strltrim(char *str)
{
	char *p,*q;

   p=q=str;
   while(isspace(*p))
   	p++;

   while(*p)
   	*q++=*p++;
   *q='\0';

	return(str);
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco               Data di creazione : 19/07/1999
 * Codice   :
 *
 * Note:	strip space on right
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

static	char *strrtrim(char *str)
{
	register int i;

	for(i=strlen(str)-1;isspace(str[i])&&i>=0;i--);
		str[i+1]='\0';

	return(str);
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco               Data di creazione : 19/07/1999
 * Codice   :
 *
 * Note:	strip space on both end
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

static	char *strtrim(char *str)
{
	return strltrim(strrtrim(str));
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco               Data di creazione : 19/07/1999
 * Codice   :
 *
 * Note:	Create directory tree
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

static	bool	createTree(char* pcsz_fileName)
{
 	char s[_MAX_PATH];
  	char drive[_MAX_DRIVE];
  	char dir[_MAX_DIR];
  	char directory[_MAX_DIR];
  	char file[_MAX_FNAME];
  	char ext[_MAX_EXT];

   if(pcsz_fileName[strlen(pcsz_fileName)-1]!='\\')
   	strcat(pcsz_fileName, "\\");

   /* split the string to separate elems */
  	_splitpath(pcsz_fileName,drive,dir,file,ext);

   strcpy(directory, "\\");
   char* p = strtok(dir, "\\");

   while(p)
   {
   	strcat(directory, p);
      strcat(directory, "\\");

      _makepath(s,drive,directory,NULL,NULL);

      if(mkdir(s))
      	return false;

		p = strtok(NULL, "\\");
   }

	return true;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco               Data di creazione : 19/07/1999
 * Codice   :
 *
 * Note:	remove separator
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

static	char*	removeSeparator(char	*psz_line)
{
	int	i_len;

	if(char*p = strchr(psz_line, sz_separator[0]))
   {
   	i_len	=	strlen(psz_line);

      i_len	-=	p-psz_line;

   	memmove(p, p+1, i_len);
   }

   return	psz_line;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco               Data di creazione : 19/07/1999
 * Codice   :
 *
 * Note:	read list file
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

static	bool	readFile(const char* pcsz_fileList, bool	b_tryToCreateParent)
{
	ifstream	inFile(pcsz_fileList);

   if(!inFile.good())
  	{
		cerr << "I/O error opening file " << pcsz_fileList << endl;
		return( false );
	}

   char	sz_line[MAX_LINELEN];
	int	i_line		=	1;
   int	i_processed	 =	0;
   int	i_created	 =	0;

   while(inFile.getline(sz_line, sizeof(sz_line)), !inFile.eof())
   {
   	i_line++;

   	if(char*p = strtrim(sz_line))
		{
			p=removeSeparator(p);

			if(p)
         {
         	i_processed++;

            if(mkdir(p))
            {
               if(b_tryToCreateParent)
               {
                  if(!createTree(p))
                     cout << "Error creating directory: " << p <<  "[" << pcsz_fileList << ":" << i_line << endl;
                  else
                     i_created++;
               }
               else
                  cout << "Error creating directory: " << p <<  "[" << pcsz_fileList << ":" << i_line << endl;
            }
            else
               i_created++;
         }
      }
   }

   cout	<<	"Processed: " << i_processed << " lines of " << pcsz_fileList << endl;
   cout	<<	i_created << " directories was created." << endl;

   return true;
}

/*
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Creazione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Creatore : Savegnago Marco               Data di creazione : 19/07/1999
 * Codice   :
 *
 * Note:	Program entry point
 *
 *>>>>>>>>>>>>>>>>>>>>>>>>>>> Revisione funzione <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
 *-------------------------------------|--------------------------------------*
 * Revisore :                                  Data di revisione :
 * Versione : 0
 *
 * Note:
 *
 */

int main (int na, char** a)
{
	bool	b_ret	=	0;
	bool	b_tryToCreateParent=false;

	// check the number of args
	// if less of 2 then print usage message

	if (na < 2)
	{
		usage();
		return -1;
	}

	SetConsoleCtrlHandler( ctrlBreakHandler, TRUE );

	for(int i=1; i<na; i++)
	{
  		char* p=a[i];

		if((*p == '-' || *p == '/' ))
		{
      	switch (toupper(*(p+1)))
			{
         	case 'C':
					++p;
					if(*(p+1))
            		sz_separator[0]	=	char(htoi(++p));
               break;

            case 'D':
         		b_tryToCreateParent=true;
               break;

				case '?':
            	usage();
					break;

				default:
            		break;
			}
		}
      else
      	if(false!=(b_ret=readFile(p, b_tryToCreateParent)))
         	break;
	}

	SetConsoleCtrlHandler( NULL, FALSE);

	return (b_ret);
}

